@@ -351,9 +351,10 @@ trailing:true, white:true, strict: false*/
         name: "navigationButtonPanel",
         classes: "xv-buttons",
         components: [
-          {kind: "onyx.Button", name: "newButton", ontap: "newItem", classes: "icon-plus"},
+          {kind: "onyx.Button", name: "newButton", ontap: "newItem", content: "_new".loc(),
+            classes: "icon-plus text"},
           {kind: "onyx.Button", name: "exportButton", ontap: "exportAttr",
-           icon: "share", content: "_export".loc(), classes: "icon-share"}
+            classes: "icon-share text", content: "_export".loc()}
         ]
       });
 
@@ -371,6 +372,10 @@ trailing:true, white:true, strict: false*/
       this.inherited(arguments);
       this.createComponents(this.buildComponents());
     },
+    allRowsSaved: function () {
+      return _.every(this.getValue().models,
+                     function (model) { return model.isReadyClean(); });
+    },
     buttonTapped: function (inSender, inEvent) {
       var editor = this.$.editableGridRow,
         model,
@@ -534,8 +539,11 @@ trailing:true, white:true, strict: false*/
       this.doExportAttr({ attr: gridbox.attr });
     },
     refreshLists: function () {
+      var collection = this.getValue();
       this.$.aboveGridList.refresh();
       this.$.belowGridList.refresh();
+      this.$.exportButton.setDisabled(! this.allRowsSaved() ||
+                                      ! collection.length);
     },
     reset: function () {
       this.setEditableIndex(null);
@@ -623,6 +631,9 @@ trailing:true, white:true, strict: false*/
       if (this.$.summaryPanel) {
         this.$.summaryPanel.setValue(model);
       }
+
+      this.$.exportButton.setDisabled(! this.allRowsSaved() ||
+                                      ! collection.length);
     }
   });
 }());
