@@ -56,6 +56,12 @@ trailing:true, white:true*/
         autoDismiss: false, modal: true, floating: true, scrim: true},
       {name: "navigator", kind: "XV.Navigator"}
     ],
+    resizeHandler: function () {
+      this.inherited(arguments);
+      if (this.$.notifyPopup.showing) {
+        this.$.notifyPopup.applyStyle("opacity", 1); // XXX not sure why this hack is necessary.
+      }
+    },
     activate: function () {
       this.goToNavigator();
       this.$.navigator.activate();
