@@ -14,8 +14,17 @@ regexp:true, undef:true, strict:true, trailing:true, white:true */
     dataSource = require('../../node-datasource/lib/ext/datasource').dataSource,
     winston = require('winston');
 
-
-
+  var defaultExtensions = [
+    path.join(__dirname, '../../foundation-database'),
+    path.join(__dirname, '../../lib/orm'),
+    path.join(__dirname, '../../enyo-client'),
+    path.join(__dirname, '../../enyo-client/extensions/source/crm'),
+    path.join(__dirname, '../../enyo-client/extensions/source/project'),
+    path.join(__dirname, '../../enyo-client/extensions/source/sales'),
+    path.join(__dirname, '../../enyo-client/extensions/source/billing'),
+    path.join(__dirname, '../../enyo-client/extensions/source/purchasing'),
+    path.join(__dirname, '../../enyo-client/extensions/source/oauth2')
+  ];
 
   var convertFromMetasql = function (content, filename, defaultSchema) {
     var lines = content.split("\n"),
@@ -61,7 +70,7 @@ regexp:true, undef:true, strict:true, trailing:true, white:true */
       tableName = defaultSchema ? defaultSchema + ".pkgreport" : "report",
       description,
       disableSql,
-      deleteSql,
+      updateSql,
       insertSql,
       enableSql;
 
@@ -80,20 +89,26 @@ regexp:true, undef:true, strict:true, trailing:true, white:true */
 
     disableSql = "ALTER TABLE " + tableName + " DISABLE TRIGGER ALL;";
 
-    deleteSql = "delete from " + tableName + " " +
-      "where report_name = '" + name +
-      "' and report_grade = " + grade + ";";
-
     insertSql = "insert into " + tableName + " (report_name, report_descrip, " +
-      "report_source, report_loaddate, report_grade) VALUES (" +
+      "report_source, report_loaddate, report_grade) select " +
       "'" + name + "'," +
       "$$" + description + "$$," +
       "$$" + content + "$$," +
-      "now(), " + grade + ");";
+      "now(), " + grade +
+      " where not exists (select c.report_id from " + tableName + " c " +
+      "where report_name = '" + name +
+      "' and report_grade = " + grade + ");";
+
+    updateSql = "update " + tableName + " set " +
+      " report_descrip = $$" + description +
+      "$$, report_source = $$" + content +
+      "$$, report_loaddate = now() " +
+      "where report_name = '" + name +
+      "' and report_grade = " + grade + ";";
 
     enableSql = "ALTER TABLE " + tableName + " ENABLE TRIGGER ALL;";
 
-    return disableSql + deleteSql + insertSql + enableSql;
+    return disableSql + insertSql + updateSql + enableSql;
   };
 
   var convertFromScript = function (content, filename, defaultSchema) {
@@ -101,25 +116,29 @@ regexp:true, undef:true, strict:true, trailing:true, white:true */
       tableName = defaultSchema ? defaultSchema + ".pkgscript" : "unknown",
       notes = "", //"xtMfg package",
       disableSql,
-      deleteSql,
       insertSql,
+      updateSql,
       enableSql;
 
     disableSql = "ALTER TABLE " + tableName + " DISABLE TRIGGER ALL;";
 
-    deleteSql = "delete from " + tableName + " " +
-      "where script_name = '" + name +
-      "';";
-
     insertSql = "insert into " + tableName + " (script_name, script_order, script_enabled, " +
-      "script_source, script_notes) VALUES (" +
+      "script_source, script_notes) select " +
       "'" + name + "', 0, TRUE, " +
       "$$" + content + "$$," +
-      "'" + notes + "');";
+      "'" + notes + "'" +
+      " where not exists (select c.script_id from " + tableName + " c " +
+      "where script_name = '" + name + "');";
+
+    updateSql = "update " + tableName + " set " +
+      "script_name = '" + name + "', script_order = 0, script_enabled = TRUE, " +
+      "script_source = $$" + content +
+      "$$, script_notes = '" + notes + "' " +
+      "where script_name = '" + name + "';";
 
     enableSql = "ALTER TABLE " + tableName + " ENABLE TRIGGER ALL;";
 
-    return disableSql + deleteSql + insertSql + enableSql;
+    return disableSql + insertSql + updateSql + enableSql;
   };
 
   var convertFromUiform = function (content, filename, defaultSchema) {
@@ -127,25 +146,28 @@ regexp:true, undef:true, strict:true, trailing:true, white:true */
       tableName = defaultSchema ? defaultSchema + ".pkguiform" : "unknown",
       notes = "", //"xtMfg package",
       disableSql,
-      deleteSql,
       insertSql,
+      updateSql,
       enableSql;
 
     disableSql = "ALTER TABLE " + tableName + " DISABLE TRIGGER ALL;";
 
-    deleteSql = "delete from " + tableName + " " +
-      "where uiform_name = '" + name +
-      "';";
-
     insertSql = "insert into " + tableName + " (uiform_name, uiform_order, uiform_enabled, " +
-      "uiform_source, uiform_notes) VALUES (" +
+      "uiform_source, uiform_notes) select " +
       "'" + name + "', 0, TRUE, " +
       "$$" + content + "$$," +
-      "'" + notes + "');";
+      "'" + notes + "' " +
+      " where not exists (select c.uiform_id from " + tableName + " c " +
+      "where uiform_name = '" + name + "');";
+
+    updateSql = "update " + tableName + " set uiform_name = '" +
+      name + "', uiform_order = 0, uiform_enabled = TRUE, " +
+      "uiform_source = $$" + content + "$$, uiform_notes = '" + notes + "' " +
+      "where uiform_name = '" + name + "';";
 
     enableSql = "ALTER TABLE " + tableName + " ENABLE TRIGGER ALL;";
 
-    return disableSql + deleteSql + insertSql + enableSql;
+    return disableSql + insertSql + updateSql + enableSql;
   };
 
   var conversionMap = {
@@ -363,6 +385,71 @@ regexp:true, undef:true, strict:true, trailing:true, white:true */
     });
   };
 
+  var pathFromExtension = function (name, location) {
+    if (location === '/core-extensions') {
+      return path.join(__dirname, "/../../enyo-client/extensions/source/", name);
+    } else if (location === '/xtuple-extensions') {
+      return path.join(__dirname, "../../../xtuple-extensions/source", name);
+    } else if (location === '/private-extensions') {
+      return path.join(__dirname, "../../../private-extensions/source", name);
+    } else if (location === 'npm') {
+      return path.join(__dirname, "../../node_modules", name);
+    }
+  };
+
+  //
+  // Looks in a database to see which extensions are registered, and
+  // tacks onto that list the core directories.
+  //
+  var inspectMobilizedDatabase = function (creds, done) {
+    var extSql = "SELECT * FROM xt.ext ORDER BY ext_load_order";
+    dataSource.query(extSql, creds, function (err, res) {
+      if (err) {
+        return done(err);
+      }
+
+      var paths = _.map(_.compact(res.rows), function (row) {
+        return pathFromExtension(row.ext_name, row.ext_location);
+      });
+
+      paths.unshift(path.join(__dirname, "../../enyo-client")); // core path
+      paths.unshift(path.join(__dirname, "../../lib/orm")); // lib path
+      paths.unshift(path.join(__dirname, "../../foundation-database")); // foundation path
+      done(null, _.compact(paths));
+    });
+  };
+
+  var inspectUnmobilizedDatabase = function (creds, done) {
+    var extSql = "select * from public.pkghead where pkghead_name in ('xtmfg', 'xwd');",
+      editionMap = {
+        xtmfg: ["inventory", "manufacturing"],
+        xwd: ["inventory", "distribution"]
+      };
+    dataSource.query(extSql, creds, function (err, res) {
+      if (err) {
+        return done(err);
+      }
+      var extensions = _.unique(_.flatten(_.map(res.rows, function (row) {
+        return _.map(editionMap[row.pkghead_name], function (ext) {
+          return path.join(__dirname, "../../../private-extensions/source", ext);
+        });
+      })));
+      done(err, defaultExtensions.concat(extensions));
+    });
+  };
+
+  var inspectDatabaseExtensions = function (creds, done) {
+    var isMobilizedSql = "select * from information_schema.tables where table_schema = 'xt' and table_name = 'ext';";
+
+    dataSource.query(isMobilizedSql, creds, function (err, res) {
+      if (res.rowCount === 0) {
+        inspectUnmobilizedDatabase(creds, done);
+      } else {
+        inspectMobilizedDatabase(creds, done);
+      }
+    });
+  };
+
   //
   // Step 0 (optional, triggered by flags), wipe out the database
   // and load it from scratch using pg_restore something.backup unless
@@ -403,7 +490,7 @@ regexp:true, undef:true, strict:true, trailing:true, white:true */
           if (err) {
             console.log("ignoring restore db error", err);
           }
-          callback(null, res);
+          done(null, res);
         });
       },
       finish = function (err, results) {
@@ -424,7 +511,15 @@ regexp:true, undef:true, strict:true, trailing:true, white:true */
       async.series([
         dropDatabase,
         createDatabase,
-        restoreBackup
+        restoreBackup,
+        function (done) {
+          credsClone.database = databaseName;
+          inspectDatabaseExtensions(credsClone, function (err, paths) {
+            // in the case of a build-from-backup, we ignore any user desires and dictate the extensions
+            spec.extensions = paths;
+            done();
+          });
+        }
       ], finish);
     }
   };
@@ -482,6 +577,9 @@ regexp:true, undef:true, strict:true, trailing:true, white:true */
       unregisterSql = ["delete from xt.usrext where usrext_id in " +
         "(select usrext_id from xt.usrext inner join xt.ext on usrext_ext_id = ext_id where ext_name = $1);",
 
+        "delete from xt.grpext where grpext_id in " +
+        "(select grpext_id from xt.grpext inner join xt.ext on grpext_ext_id = ext_id where ext_name = $1);",
+
         "delete from xt.clientcode where clientcode_id in " +
         "(select clientcode_id from xt.clientcode inner join xt.ext on clientcode_ext_id = ext_id where ext_name = $1);",
 
@@ -511,6 +609,8 @@ regexp:true, undef:true, strict:true, trailing:true, white:true */
     async.each(specs, unregisterEach, masterCallback);
   };
 
+  exports.defaultExtensions = defaultExtensions;
+  exports.inspectDatabaseExtensions = inspectDatabaseExtensions;
   exports.explodeManifest = explodeManifest;
   exports.initDatabase = initDatabase;
   exports.sendToDatabase = sendToDatabase;
